/*
-----------------------------------------------------------------------------
 ==== 3OS kernel module ===
 T   ++ 
-----------------------------------------------------------------------------
*/

#include "kernel.hpp"
#include "mem.hpp"
#include "heap.hpp"
#include "spaces.hpp"
#include "stream.hpp"
#include "cpuinfo.hpp"
#include "hardware.hpp"
#include "lkm\lkm.hpp"

extern c_Kernel  *pc_Krnl3OS;
extern char LKM_example[200];
extern char KrnlStream[84];
extern s_DLCL DLCLStorage[2];

void* _DClassLink(int nClass)
{
 return DLCLStorage[nClass].vObject;
}

void c_Kernel::Stabilize(void){
 pc_Out = new c_Out(vBaseAddr);
 pc_Out->printf("PM > \001\x47 3OS Kernel \001\x2F allocated %d \n\r", (dword)GlobalAddr(0));
 pc_GDT = new c_GDT();
 pc_Out->printf("GDT located %d \n\r", pc_GDT->AddrGet());
 pc_GDT->ReLocate();
 pc_Out->printf("GDT relocated %d \n\r", pc_GDT->AddrGet());

 pc_SpcCode = (c_CodeSpace*) pc_GDT->Dsc_Get(1);
 pc_PIC = new c_PIC(0x20);
 pc_IDT = new c_IDT();


#ifdef CPU_DETECT
 CCpuInfo *CpuInfo = new CCpuInfo();

 pc_Out->puts("CPU Vendor Id: ");
 pc_Out->puts(CpuInfo->GetVendorId());
 if(CpuInfo->GetStdFeatures() & SFF_FPU)  pc_Out->puts("\n\rCPU Features: Built-in FPU");
 if(CpuInfo->GetStdFeatures() & SFF_MMX)  pc_Out->puts(", MMX");
 if(CpuInfo->GetStdFeatures() & SFF_SSE)  pc_Out->puts(", SSE");
 if(CpuInfo->GetStdFeatures() & SFF_SSE2) pc_Out->puts(", SSE2");
 if(CpuInfo->GetExtAMDFeatures() & AFF_3DNOW) pc_Out->puts(", 3DNOW!");
 if(CpuInfo->GetExtAMDFeatures() & AFF_3DNOWEX) pc_Out->puts(", Ext. 3DNOW!");
 pc_Out->puts("\n\r");
#endif

#ifdef SPACE_TEST
 pc_Out->puts("Test space construction\n\r");

 c_DataSpace *pc_TestSpace = new c_DataSpace(80*25*2, k_KLvl, (void*) 0xb8000);
 _asm{
  push es
   mov  ax, 0x28
   mov  es, ax
   mov  edi, 960
   mov  eax, 0x05440544
   mov  ecx, 160
   rep stosd
  pop  es
 }
#endif

#ifdef STREAM_LOAD
 pc_Out->puts("Threads stream ");

 if((*(dword*) KrnlStream) == 'TRDS')
  pc_Out->puts("\001\x2F found!\n\r");
 else
  pc_Out->puts("\001\x47 not found!\n\r");

 c_ThreadStream *pc_Stream = new c_ThreadStream((s_Stream*) &KrnlStream[4]);
 pc_Out->printf("Stream size %d, code %d\r\n",pc_Stream->Size(),pc_Stream->CntCodeSec());
#endif

#ifdef LKM_TEST
 c_LKM *pc_LKM = (c_LKM*) &LKM_example;
 if(pc_LKM->IsLKM()){
  pc_Out->puts("LKM found!");
  pc_LKM->Start(_DClassLink);
 }
#endif

#ifdef SWITCH_TEST
 c_Thread *pc_Th, *pc_ShareTh;
 pc_ThList = new c_ThList();

/* ⠢  ।  ᮧ । N1 */
 pc_Th = pc_ThList->Add(new c_Item(new c_Thread(k_High, k_KLvl)))->Item;
/* 㧨  ⮪ । */
 pc_Th->LoadFromStream(pc_Stream);
 pc_Th->SetESI(0xB8000);

for(int i=4;i<180; i+=4){
/* ᮧ । Ni */
 pc_ShareTh = new c_Thread(k_High, k_KLvl);
/* ਬ */
 pc_ShareTh->Shared(pc_Th);
 pc_ShareTh->SetESI(0xB8000+i);
/* ⠢  । ७ । N2 */
 pc_ThList->Add(new c_Item(pc_ShareTh));
}

 pc_ThList->Imdtly_Get()->Switch();
#endif

m1:
 m_interrupt_On();
 goto m1;
}

void c_Kernel::ICorruption(void){
// pc_Out->printf("Interrupt corruption vector!");
 pc_Out->gotoxy(1,20);
 pc_Out->printf("List = %d threads, Imdtly=%d \001\x47timer:\001\x1F %d", pc_ThList->Count(), pc_ThList->Imdtly_Get(), tcount++);
 pc_PIC->ResetIRS();
}

void _ICorruption(void){
#pragma aux _ICorruption modify [eax ebx ecx edx esi edi ebp esp];
 push_int();

 SETSEG(DS, k_GDKrnlData);      //  
 SETSEG(ES, k_GDKrnlData);      //  
 SETSEG(FS, k_GDKrnlFlat);      // 4Gb ᥣ 
 pc_Krnl3OS->ICorruption();

 pc_Krnl3OS->pc_ThList->Rotate();
 pc_Krnl3OS->pc_ThList->Imdtly_Get()->Switch();

 pop_int();
 Int_ret();
}