#ifndef __SPACES
/*
 === 3OS Kernel definition module class spaces ===
     3
*/
#define __SPACES
#include "descript.hpp"
#include "hardware.hpp"
#include "heap.hpp"

extern THeap *pc_Heap;

#pragma pack(1)

class c_Space : public c_DSpace{
 public:
   c_Space(void *vMem, dword nSize, e_DPLevel e_Lvl, e_SpcType e_SpType);
   c_Space(dword nSize, e_DPLevel e_Lvl, e_SpcType e_SpType);
   ~c_Space() {};

   void ReSize(dword nSize)   {};  //  
   void ReLocate(void *vAddr) {};  //   
};

class c_DataSpace : public c_Space{
 public:
  c_DataSpace(dword nSize, e_DPLevel e_Lvl)
   : c_Space(nSize, e_Lvl, k_RWDataUp) {};
  c_DataSpace(dword nSize, e_DPLevel e_Lvl, void *vMem)
   : c_Space(vMem, nSize, e_Lvl, k_RWDataUp) {};

  ~c_DataSpace() {};
};

class c_CodeSpace : public c_Space{
 public:
  c_CodeSpace(dword nSize, e_DPLevel e_Lvl)
   : c_Space(nSize, e_Lvl, k_ERCode) {};
  c_CodeSpace(dword nSize, e_DPLevel e_Lvl, void *vMem)
   : c_Space(vMem, nSize, e_Lvl, k_ERCode) {};

  ~c_CodeSpace() {};
};

class c_StackSpace : public c_Space{
 public:
  c_StackSpace(dword nSize, e_DPLevel e_Lvl)
   : c_Space(nSize, e_Lvl, k_RWDataUp) {};
  c_StackSpace(dword nSize, e_DPLevel e_Lvl, void *vMem)
   : c_Space(vMem, nSize, e_Lvl, k_RWDataUp) {};

  ~c_StackSpace() {};
};

struct s_PStack{
 dword  wESP;
 word   wSS;
 word   Reserv;
};

struct s_STDTSS{
 word   wBackLink;
 word   wRes0;
 s_PStack       s_KrnlStack;
 s_PStack       s_DrvStack;
 s_PStack       s_SrvStack;
 dword  dwCR3;
 dword  dwEIP;
 dword  dwEFLAGS;
 dword  dwEAX;
 dword  dwECX;
 dword  dwEDX;
 dword  dwEBX;
 dword  dwESP;
 dword  dwEBP;
 dword  dwESI;
 dword  dwEDI;
 word   wES;
 word   wRes1;
 word   wCS;
 word   wRes2;
 word   wSS;
 word   wRes3;
 word   wDS;
 word   wRes4;
 word   wFS;
 word   wRes5;
 word   wGS;
 word   wRes6;
 word   wLDT;
 dword  dwRes7;
 word   wOffsBitMapIO;

 void TrapBit_Set(void) {dwRes7 |= 0x00010000;};
 void TrapBit_Clr(void) {dwRes7 &= 0xfffeffff;};
};

typedef void (SWCALL)(void);

struct s_JmpFar{
 byte  bOpcode;
 dword dwOffset;
 word  wSelector;
 byte  bRetOp;
};

class c_Context{
 public:
  s_STDTSS  s_TSS;
  c_DSpace  *pc_DSpace;

 protected:
  s_JmpFar  s_Switch;

 public:
  c_Context(e_DPLevel e_Lvl);
  ~c_Context() {};

  void Clear(void);
  void Switch(void)      {((SWCALL*)&s_Switch)();};
  void SetESI(dword val) {s_TSS.dwESI = val;};
};

#pragma pack(0)

#endif